#define VERSION 1.13
// Desmartins Daniel 07/10/2023 // ENGAGE PUMA

#include <mcp_can.h>
#include <SPI.h>

const bool activeIs = LOW;

#define STEER_AUX_PIN 3
#define WORK_AUX_PIN 4
#define STEER_ENGAGE_PIN 6

#define CAN0_INT 9                    // Set INT to pin 9
MCP_CAN CAN0(10);                     // Set CS to pin 10

//Variables
bool lastTrigger = false;
bool txLevel = false;
bool lastTxLevel = false;

bool steerDrive = false;
bool MCP_Run = false;

long unsigned int rxId;
unsigned char len = 0;
unsigned char rxBuf[8];

/*uint8_t currentData[8] = {0xCA, 0x00, 0xC1, 0xF0, 0xFF, 0xC0, 0x0F, 0xF0}; //Set Command Data
uint8_t lastPositionRelevageSend = currentData[2];
//0xCA = 202 ; 0x82 = 130 ; 0x9A = 154 ; 0xB2 = 178 ;*/

long unsigned int lastIdActive = 0;

void setup()
{
  pinMode(STEER_AUX_PIN, OUTPUT);
  pinMode(WORK_AUX_PIN, OUTPUT);
  digitalWrite(STEER_AUX_PIN, !activeIs);
  digitalWrite(WORK_AUX_PIN, !activeIs);

  pinMode(CAN0_INT, INPUT);

  delay(300);
  Serial.begin(38400);
  Serial.println("Firmware : EngagePuma");
  Serial.print("Version : ");
  Serial.println(VERSION);
  if (CAN0.begin(MCP_ANY, CAN_250KBPS, MCP_8MHZ) == CAN_OK) {
    Serial.println("MCP2515 Initialized Successfully!");
    Serial.println("MCP2515 Library Mask & Filter Example...");
    CAN0.init_Mask(0,1,0xFFFFFF);         // Init first mask...    
    CAN0.init_Filt(0,1,0x14FF7706);       // Init first filter...  
    CAN0.init_Filt(1,1,0x14FF7706);
    
    CAN0.init_Mask(1,1,0xFFFFFF);         // Init second mask... 
    CAN0.init_Filt(2,1,0x18FFB306);       // Init third filter...
    CAN0.init_Filt(3,1,0x18FFB031);       // Init fouth filter...
    CAN0.init_Filt(4,1,0x18FFB031);       // Init fifth filter...
    CAN0.init_Filt(5,1,0x18FFB031);       // Init sixth filter...*/
    
    CAN0.setMode(MCP_NORMAL);                // Change to normal mode to allow messages to be transmitted

    MCP_Run = true;
  } else {
    Serial.println("Error Initializing MCP2515...");
  }  
}

void loop()
{
  if(MCP_Run && !digitalRead(CAN0_INT))
  {
    CAN0.readMsgBuf(&rxId, &len, rxBuf); // Read data: len = data length, buf = data byte(s)
    
    if ((digitalRead(STEER_AUX_PIN) == activeIs) && rxId != lastIdActive) return;
    
    if (rxId == 0x94FF7706) {
      //CAN0.sendMsgBuf(0x14FF7706, 1, 8, currentData);
      if (rxBuf[6] == 0x0F) {
        if (rxBuf[1] == 0x01) {
          Serial.println("0x94FF7706 Bouton Autosteer");
          digitalWrite(STEER_AUX_PIN, activeIs);
          lastIdActive = rxId;
        } else if (rxBuf[1] == 0x04) {
          Serial.println("0x94FF7706 Gachette");
          digitalWrite(STEER_AUX_PIN, activeIs);
          lastIdActive = rxId;
        } else {
          digitalWrite(STEER_AUX_PIN, !activeIs);
          lastIdActive = 0;
        }

        if (rxBuf[2] == 0xC1) {
          Serial.println("0x94FF7706 Bouton Relevage Montée");
          digitalWrite(WORK_AUX_PIN, activeIs);
        } else if (rxBuf[2] == 0xC4) {
          Serial.println("0x94FF7706 Bouton Relevage Descendre");
          digitalWrite(WORK_AUX_PIN, !activeIs);
        }
      }
    } else if (rxId == 0x98FFB306) {
      //CAN0.sendMsgBuf(0x18FFB706, 1, 8, currentData);
      if (rxBuf[2] == 0x1F) {
        Serial.println("0x98FFB306 Bouton Gachette");
        //CAN0.sendMsgBuf(0x1CE69306, 1, 8, currentData);
        digitalWrite(STEER_AUX_PIN, activeIs);
        lastIdActive = rxId;
      } else {
        digitalWrite(STEER_AUX_PIN, !activeIs);
        lastIdActive = 0;
      }
    }
    else if (rxId == 0x98FFB031) {
      if (rxBuf[2] == 0xD0) {
        Serial.println("0x98FFB031 Bouton Autosteer");
        //CAN0.sendMsgBuf(0x1CE69031, 1, 8, currentData);
        digitalWrite(STEER_AUX_PIN, activeIs);
        lastIdActive = rxId;
      } else {
        digitalWrite(STEER_AUX_PIN, !activeIs);
        lastIdActive = 0;
      }
    }
  }
}

/*void setRelevage(bool raise_t)
{
  if (raise_t)
    currentData[2] = 0xC1;
  else
    //currentData[2] = 0xC4;
  
  //if (lastPositionRelevageSend != currentData[2]) {
    currentData[0] = 0xCA;
    CAN0.sendMsgBuf(0x14FF7706, 1, 8, currentData);
    delay(1);
    currentData[0] = 0x82;
    CAN0.sendMsgBuf(0x14FF7706, 1, 8, currentData);
    delay(1);
    currentData[0] = 0x9A;
    CAN0.sendMsgBuf(0x14FF7706, 1, 8, currentData);
    delay(1);
    currentData[0] = 0xB2;
    CAN0.sendMsgBuf(0x14FF7706, 1, 8, currentData);
    
    lastPositionRelevageSend = currentData[2];
    delay(1000);
  //}
}*/
//0xCA = 202 ; 0x82 = 130 ; 0x9A = 154 ; 0xB2 = 178 ;
